@@ -25,6 +25,9 @@
 #' @param combined_chlorine Combined chlorine (chloramines) in mg/L as Cl2. Used when a starting water has a chloramine residual.
 #' @param tot_po4 Phosphate in mg/L as PO4 3-. Used when a starting water has a phosphate residual.
 #' @param tot_nh3 Total ammonia in mg/L as N
+#' @param tot_bo3 Total borate (B(OH)4 -) in mg/L as B
+#' @param tot_sio4 Total silicate in mg/L as SiO2
+#' @param tot_ch3coo Total acetate in mg/L
 #' @param tds Total Dissolved Solids in mg/L (optional if ions are known)
 #' @param cond Electrical conductivity in uS/cm (optional if ions are known)
 #' @param toc Total organic carbon (TOC) in mg/L
@@ -52,7 +55,13 @@
 #'   \item{cond}{electrical conductivity, numeric, uS/cm.}
 #'   \item{tot_hard}{total hardness, numeric, mg/L as CaCO3.}
 #'   \item{kw}{dissociation constant for water, numeric, unitless.}
-#'   \item{alk_eq}{alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{alk_eq}{total alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{carbonate_alk_eq}{carbonate alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{phosphate_alk_eq}{phosphate alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{ammonium_alk_eq}{ammonium alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{borate_alk_eq}{borate alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{silicate_alk_eq}{silicate alkalinity as equivalents, numeric, equivalent (eq).}
+#'   \item{hypochlorite_alk_eq}{hypochlorite alkalinity as equivalents, numeric, equivalent (eq).}
 #'   \item{toc}{total organic carbon, numeric, mg/L.}
 #'   \item{doc}{dissolved organic carbon, numeric, mg/L.}
 #'   \item{bdoc}{biodegradable organic carbon, numeric, mg/L.}
@@ -73,11 +82,18 @@
 #'   \item{hpo4}{hydrogen phosphate, numeric, mols/L.}
 #'   \item{po4}{phosphate, numeric, mols/L.}
 #'   \item{nh4}{ammonium, numeric, mol/L as N.}
+#'   \item{bo3}{borate, numeric, mol/L.}
+#'   \item{h3sio4}{trihydrogen silicate, numeric, mol/L.}
+#'   \item{h2sio4}{dihydrogen silicate, numeric, mol/L.}
+#'   \item{ch3coo}{acetate, numeric, mol/L.}
 #'   \item{h}{hydrogen ion, numeric, mol/L.}
 #'   \item{oh}{hydroxide ion, numeric, mol/L.}
 #'   \item{tot_po4}{total phosphate, numeric, mol/L.}
 #'   \item{tot_nh3}{total ammonia, numeric, mol/L.}
 #'   \item{tot_co3}{total carbonate, numeric, mol/L.}
+#'   \item{tot_bo3}{total borate, numeric, mol/L.}
+#'   \item{tot_sio4}{total silicate, numeric, mol/L.}
+#'   \item{tot_ch3coo}{total acetate, numeric, mol/L.}
 #'   \item{br}{bromide, numeric, mol/L.}
 #'   \item{bro3}{bromate, numeric, mol/L.}
 #'   \item{f}{fluoride, numeric, mol/L.}
@@ -108,8 +124,8 @@
 #' }
 
 define_water <- function(ph, temp = 25, alk, tot_hard, ca, mg, na, k, cl, so4, mno4,
-                         free_chlorine = 0, combined_chlorine = 0, tot_po4 = 0, tot_nh3 = 0, tds, cond,
-                         toc, doc, uv254, br, f, fe, al, mn, no3) {
+                         free_chlorine = 0, combined_chlorine = 0, tot_po4 = 0, tot_nh3 = 0, tot_ch3coo = 0, tot_bo3 = 0, tot_sio4 = 0,
+                         tds, cond, toc, doc, uv254, br, f, fe, al, mn, no3) {
   # Initialize string for tracking which parameters were estimated
   estimated <- ""
 
@@ -180,6 +196,9 @@ define_water <- function(ph, temp = 25, alk, tot_hard, ca, mg, na, k, cl, so4, m
   free_chlorine <- convert_units(free_chlorine, "cl2")
   combined_chlorine <- convert_units(combined_chlorine, "cl2")
   tot_nh3 <- convert_units(tot_nh3, "n")
+  tot_bo3 <- convert_units(tot_bo3, "b")
+  tot_sio4 <- convert_units(tot_sio4, "sio2")
+  tot_ch3coo <- convert_units(tot_ch3coo, "ch3cooh")
 
   br <- ifelse(missing(br), NA_real_, convert_units(br, "br", "ug/L", "M"))
   f <- ifelse(missing(f), NA_real_, convert_units(f, "f"))
@@ -221,33 +240,72 @@ define_water <- function(ph, temp = 25, alk, tot_hard, ca, mg, na, k, cl, so4, m
   oh <- kw / h # assume activity = concentration to start
 
   # convert alkalinity input to equivalents/L
-  carb_alk_eq <- convert_units(alk, "caco3", startunit = "mg/L CaCO3", endunit = "eq/L")
-  # calculate total carbonate concentration
+  alk_eq <- convert_units(alk, "caco3", startunit = "mg/L CaCO3", endunit = "eq/L")
   # Initial alpha values (not corrected for IS)
   discons <- tidywater::discons
+
   k1co3 <- K_temp_adjust(discons["k1co3", ]$deltah, discons["k1co3", ]$k, temp)
   k2co3 <- K_temp_adjust(discons["k2co3", ]$deltah, discons["k2co3", ]$k, temp)
+  k1po4 <- K_temp_adjust(discons["k1po4", ]$deltah, discons["k1po4", ]$k, temp)
+  k2po4 <- K_temp_adjust(discons["k2po4", ]$deltah, discons["k2po4", ]$k, temp)
+  k3po4 <- K_temp_adjust(discons["k3po4", ]$deltah, discons["k3po4", ]$k, temp)
+  kocl <- K_temp_adjust(discons["kocl", ]$deltah, discons["kocl", ]$k, temp)
+  knh4 <- K_temp_adjust(discons["knh4", ]$deltah, discons["knh4", ]$k, temp)
+  kbo3 <- K_temp_adjust(discons["kbo3", ]$deltah, discons["kbo3", ]$k, temp)
+  k1sio4 <- K_temp_adjust(discons["k1sio4", ]$deltah, discons["k1sio4", ]$k, temp)
+  k2sio4 <- K_temp_adjust(discons["k2sio4", ]$deltah, discons["k2sio4", ]$k, temp)
 
   alpha0 <- calculate_alpha0_carbonate(h, data.frame("k1co3" = k1co3, "k2co3" = k2co3)) # proportion of total carbonate as H2CO3
   alpha1 <- calculate_alpha1_carbonate(h, data.frame("k1co3" = k1co3, "k2co3" = k2co3)) # proportion of total carbonate as HCO3-
   alpha2 <- calculate_alpha2_carbonate(h, data.frame("k1co3" = k1co3, "k2co3" = k2co3)) # proportion of total carbonate as CO32-
-  tot_co3 <- (carb_alk_eq + h - oh) / (alpha1 + 2 * alpha2)
+
+  alpha1p <- calculate_alpha1_phosphate(h, data.frame("k1po4" = k1po4, "k2po4" = k2po4, "k3po4" = k3po4)) # proportion of total phosphate as H2PO4-
+  alpha2p <- calculate_alpha2_phosphate(h, data.frame("k1po4" = k1po4, "k2po4" = k2po4, "k3po4" = k3po4)) # proportion of total phosphate as HPO4 2-
+  alpha3p <- calculate_alpha3_phosphate(h, data.frame("k1po4" = k1po4, "k2po4" = k2po4, "k3po4" = k3po4)) # proportion of total phosphate as PO4 3-
+
+  alpha1c <- calculate_alpha1_hypochlorite(h, data.frame("kocl" = kocl))
+  alpha1n <- calculate_alpha1_ammonia(h, data.frame("knh4" = knh4))
+  alpha1b <- calculate_alpha1_borate(h, data.frame("kbo3" = kbo3))
+  alpha1s <- calculate_alpha1_silicate(h, data.frame("k1sio4" = k1sio4, "k2sio4" = k2sio4))
+  alpha2s <- calculate_alpha2_silicate(h, data.frame("k1sio4" = k1sio4, "k2sio4" = k2sio4))
+
+  # Update total ion values
+  h2po4 <- tot_po4 * alpha1p
+  hpo4 <- tot_po4 * alpha2p
+  po4 <- tot_po4 * alpha3p
+  h3po4 <- tot_po4 - (h2po4 + hpo4 + po4)
+  ocl <- free_chlorine * alpha1c
+  nh4 <- tot_nh3 * alpha1n
+
+  bo3 <- tot_bo3 * alpha1b
+  h3sio4 <- tot_sio4 * alpha1s
+  h2sio4 <- tot_sio4 * alpha2s
+
+  phosphate_alk_eq <- (-1 * h3po4 + 0 * h2po4 + 1 * hpo4 + 2 * po4)
+  hypochlorite_alk_eq <- (1 * ocl)
+  ammonium_alk_eq <- (1 * nh4)
+  borate_alk_eq <- (1 * bo3)
+  silicate_alk_eq <- (1 * h3sio4 + 2 * h2sio4)
+  carbonate_alk_eq <- alk_eq - (ammonium_alk_eq + borate_alk_eq + phosphate_alk_eq + silicate_alk_eq + hypochlorite_alk_eq + oh) + h
+
+  tot_co3 <- carbonate_alk_eq / (alpha1 + 2 * alpha2)
 
   # Initialize water to simplify IS calcs
   water <- methods::new("water",
     ph = ph, temp = temp, alk = alk, tds = tds, cond = cond, tot_hard = tot_hard,
     na = na, ca = ca, mg = mg, k = k, cl = cl, so4 = so4, mno4 = mno4,
     h2co3 = tot_co3 * alpha0, hco3 = tot_co3 * alpha1, co3 = tot_co3 * alpha2,
-    h2po4 = 0, hpo4 = 0, po4 = 0, ocl = 0, nh4 = 0,
+    h2po4 = h2po4, hpo4 = hpo4, po4 = po4, ocl = ocl, nh4 = nh4,
+    bo3 = bo3, h3sio4 = h3sio4, h2sio4 = h2sio4,
     h = h, oh = oh,
     tot_po4 = tot_po4, free_chlorine = free_chlorine, combined_chlorine = combined_chlorine, tot_nh3 = tot_nh3, tot_co3 = tot_co3,
-    kw = kw, is = 0, alk_eq = carb_alk_eq,
+    tot_bo3 = tot_bo3, tot_sio4 = tot_sio4, tot_ch3coo = tot_ch3coo,
+    kw = kw, is = 0, alk_eq = alk_eq,
     doc = doc, toc = toc, uv254 = uv254,
     br = br, f = f, fe = fe, al = al, mn = mn, no3 = no3
   )
 
   # Determine ionic strength
-
   if (!is.na(tds)) {
     water@is <- correlate_ionicstrength(tds, from = "tds")
     water@cond <- correlate_ionicstrength(tds, from = "tds", to = "cond")
@@ -281,12 +339,8 @@ define_water <- function(ph, temp = 25, alk, tot_hard, ca, mg, na, k, cl, so4, m
   alpha0 <- calculate_alpha0_carbonate(h, ks)
   alpha1 <- calculate_alpha1_carbonate(h, ks) # proportion of total carbonate as HCO3-
   alpha2 <- calculate_alpha2_carbonate(h, ks) # proportion of total carbonate as CO32-
-  water@tot_co3 <- (carb_alk_eq + h - oh) / (alpha1 + 2 * alpha2)
-  water@h2co3 <- water@tot_co3 * alpha0
-  water@hco3 <- water@tot_co3 * alpha1
-  water@co3 <- water@tot_co3 * alpha2
-  water@dic <- water@tot_co3 * tidywater::mweights$dic * 1000
 
+  alpha0p <- calculate_alpha0_phosphate(h, ks)
   alpha1p <- calculate_alpha1_phosphate(h, ks)
   alpha2p <- calculate_alpha2_phosphate(h, ks)
   alpha3p <- calculate_alpha3_phosphate(h, ks)
@@ -294,12 +348,29 @@ define_water <- function(ph, temp = 25, alk, tot_hard, ca, mg, na, k, cl, so4, m
   water@h2po4 <- tot_po4 * alpha1p
   water@hpo4 <- tot_po4 * alpha2p
   water@po4 <- tot_po4 * alpha3p
+  h3po4 <- tot_po4 * alpha0p
 
   water@ocl <- free_chlorine * calculate_alpha1_hypochlorite(h, ks)
   water@nh4 <- tot_nh3 * calculate_alpha1_ammonia(h, ks)
+  water@ch3coo <- tot_ch3coo * calculate_alpha1_acetate(h, ks)
 
-  # Calculate total alkalinity (set equal to carbonate alkalinity for now)
-  water@alk_eq <- carb_alk_eq
+  water@bo3 <- tot_bo3 * calculate_alpha1_borate(h, ks)
+  water@h3sio4 <- tot_sio4 * calculate_alpha1_silicate(h, ks)
+  water@h2sio4 <- tot_sio4 * calculate_alpha2_silicate(h, ks)
+
+  # Calculate individual and total alkalinity
+  water@phosphate_alk_eq <- (-1 * h3po4 + 0 * water@h2po4 + 1 * water@hpo4 + 2 * water@po4)
+  water@hypochlorite_alk_eq <- (1 * water@ocl)
+  water@ammonium_alk_eq <- (1 * water@nh4)
+  water@borate_alk_eq <- (1 * water@bo3)
+  water@silicate_alk_eq <- (1 * water@h3sio4 + 2 * water@h2sio4)
+  water@carbonate_alk_eq <- alk_eq - (water@ammonium_alk_eq + water@borate_alk_eq + water@phosphate_alk_eq + water@silicate_alk_eq + hypochlorite_alk_eq + water@oh) + water@h
+
+  water@tot_co3 <- water@carbonate_alk_eq / (alpha1 + 2 * alpha2)
+  water@h2co3 <- water@tot_co3 * alpha0
+  water@hco3 <- water@tot_co3 * alpha1
+  water@co3 <- water@tot_co3 * alpha2
+  water@dic <- water@tot_co3 * tidywater::mweights$dic * 1000
 
   # Add all estimated values to water slot
   water@estimated <- estimated
@@ -307,96 +378,54 @@ define_water <- function(ph, temp = 25, alk, tot_hard, ca, mg, na, k, cl, so4, m
   return(water)
 }
 
-#' Apply `define_water` and output a dataframe
-#'
-#' This function allows \code{\link{define_water}} to be added to a piped data frame.
-#' It outputs all carbonate calculations and other parameters in a data frame.
-#' tidywater functions cannot be added after this function because they require a `water` class input.
-#'
-#'  For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
-#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
-#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
-#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
-#'  shorter run times will not benefit from parallel processing.
-#'
-#' @param df a data frame containing columns with all the parameters listed in \code{\link{define_water}}
-#'
-#' @seealso \code{\link{define_water}}
-#'
-#' @examples
-#'
-#' example_df <- water_df %>%
-#'   define_water_once()
-#'
-#' @import dplyr
-#' @importFrom tidyr unnest_wider
-#' @export
-#' @returns A data frame containing columns that were filled or calculated based on define_water.
-
-define_water_once <- function(df) {
-  defined_df <- defined_water <- NULL # Quiet RCMD check global variable note
-  df %>%
-    define_water_chain() %>%
-    mutate(defined_df = furrr::future_map(defined_water, convert_water)) %>%
-    unnest_wider(defined_df) %>%
-    select(-defined_water) %>%
-    as.data.frame()
-}
-
 #' Apply `define_water` within a dataframe and output a column of `water` class to be chained to other tidywater functions
 #'
-#' This function allows \code{\link{define_water}} to be added to a piped data frame.
+#' This function allows [define_water] to be added to a piped data frame.
 #' Its output is a `water` class, and can therefore be chained with "downstream" tidywater functions.
 #'
-#'  For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
-#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
-#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
-#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
-#'  shorter run times will not benefit from parallel processing.
+#' @param df a data frame containing columns with all the desired parameters with column names matching argument names in define_water
+#' @param output_water name of the output column storing updated parameters with the class, water. Default is "defined".
+#' @param pluck_cols Extract primary water slots (ph, alk, doc, uv254) into new numeric columns for easy access. Default to FALSE.
+#' @param water_prefix Append the output_water name to the start of the plucked columns. Default is TRUE.
 #'
-#' @param df a data frame containing columns with all the parameters listed in \code{\link{define_water}}
-#' @param output_water name of the output column storing updated parameters with the class, water. Default is "defined_water".
-#'
-#' @seealso \code{\link{define_water}}
+#' @seealso [define_water]
 #'
 #' @examples
 #' \donttest{
 #' example_df <- water_df %>%
-#'   define_water_chain() %>%
-#'   balance_ions_chain()
+#'   define_water_df() %>%
+#'   balance_ions_df()
 #'
 #' example_df <- water_df %>%
-#'   define_water_chain(output_water = "This is a column of water") %>%
-#'   balance_ions_chain(input_water = "This is a column of water")
-#'
-#' # Initialize parallel processing
-#' library(furrr)
-#' # plan(multisession)
-#' example_df <- water_df %>%
-#'   define_water_chain() %>%
-#'   balance_ions_chain()
-#'
-#' #' #Optional: explicitly close multisession processing
-#' # plan(sequential)
+#'   define_water_df(output_water = "This is a column of water") %>%
+#'   balance_ions_df(input_water = "This is a column of water")
 #' }
 #'
-#' @import dplyr
 #' @export
 #' @returns A data frame containing a water class column.
 
-define_water_chain <- function(df, output_water = "defined_water") {
+define_water_df <- function(df, output_water = "defined", pluck_cols = FALSE, water_prefix = TRUE) {
   define_water_args <- c(
-    "ph", "temp", "alk", "tot_hard", "ca", "mg", "na", "k", "cl", "so4", "mno4", "free_chlorine", "combined_chlorine", "tot_po4", "tot_nh3",
+    "ph", "temp", "alk", "tot_hard", "ca", "mg", "na", "k", "cl", "so4", "mno4", "free_chlorine", "combined_chlorine", "tot_po4", "tot_nh3", "tot_ch3coo",
     "tds", "cond",
     "toc", "doc", "uv254", "br", "f", "fe", "al", "mn"
   )
 
-  extras <- df %>%
-    select(!any_of(define_water_args))
+  water_input <- df[, names(df) %in% define_water_args]
 
-  output <- df %>%
-    select(any_of(define_water_args)) %>%
-    mutate(!!output_water := furrr::future_pmap(., define_water)) %>%
-    select(!any_of(define_water_args)) %>%
-    cbind(extras)
+  df[[output_water]] <- lapply(seq_len(nrow(df)), function(i) {
+    do.call(define_water, water_input[i, ])
+  })
+
+  output <- df[, !names(df) %in% define_water_args, drop = FALSE]
+
+  if (pluck_cols) {
+    output <- output |>
+      pluck_water(c(output_water), c("ph", "alk", "doc", "uv254"))
+    if (!water_prefix) {
+      names(output) <- gsub(paste0(output_water, "_"), "", names(output))
+    }
+  }
+
+  return(output)
 }
