@@ -15,6 +15,8 @@
 #
 
 ###############
+library(dplyr)
+
 test_that("chemdose_chloramine returns free_chlorine = 0 when existing free chlorine in the system and chlorine dose are 0.", {
   water1 <- suppressWarnings(define_water(7.5, 21, 66))
   water2 <- suppressWarnings(chemdose_chloramine(water1, time = 20))
@@ -97,9 +99,9 @@ test_that("chemdose_chloramine uses slot only when dose is zero or missing or us
   expect_equal(water4@nh2cl, water5@nh2cl)
 })
 
-# chemdose_chloramine_chain ----
-# Test that chemdose_chloramine_chain outputs are the same as base function, chemdose_chloramine.
-test_that("chemdose_chloramine_chain outputs the same as base, chemdose_chloramine", {
+# chemdose_chloramine_df ----
+# Test that chemdose_chloramine_df outputs are the same as base function, chemdose_chloramine.
+test_that("chemdose_chloramine_df outputs the same as base, chemdose_chloramine", {
   testthat::skip_on_cran()
   water0 <- define_water(
     ph = 7.9, temp = 20, alk = 50, tot_hard = 50, ca = 13, mg = 4, na = 20, k = 20,
@@ -110,13 +112,20 @@ test_that("chemdose_chloramine_chain outputs the same as base, chemdose_chlorami
 
   water2 <- water_df %>%
     slice(1) %>%
-    define_water_chain() %>%
-    chemdose_chloramine_chain(nh3 = 1, cl2 = 1, time = 20) %>%
-    pluck_water(c("chlorinated_water"), c("ph", "free_chlorine", "combined_chlorine"))
+    define_water_df() %>%
+    chemdose_chloramine_df(nh3 = 1, cl2 = 1, time = 20) %>%
+    pluck_water(c("chloraminated"), c("free_chlorine", "nh2cl", "nhcl2", "ncl3", "combined_chlorine", "tot_nh3"))
 
-  expect_equal(water2$chlorinated_water_ph[1], water1@ph)
-  expect_equal(water2$chlorinated_water_free_chlorine[1], water1@free_chlorine)
-  expect_equal(water2$chlorinated_water_combined_chlorine[1], water1@combined_chlorine)
+  # check that pluck_cols does the same thing as pluck_Water
+  water3 <- water_df %>%
+    slice(1) %>%
+    define_water_df() %>%
+    chemdose_chloramine_df(nh3 = 1, cl2 = 1, time = 20, pluck_cols = TRUE)
+
+  expect_equal(water2$chloraminated_free_chlorine[1], water1@free_chlorine)
+  expect_equal(water2$chloraminated_combined_chlorine[1], water1@combined_chlorine)
+  expect_equal(ncol(water2), ncol(water3))
+  expect_equal(water2$chloraminated_combined_chlorine[1], water3$chloraminated_combined_chlorine[1])
 
   water3 <- suppressWarnings(define_water(
     ph = 7.9, temp = 20, alk = 50, tot_hard = 50, ca = 13, mg = 4, na = 20, k = 20,
@@ -128,130 +137,110 @@ test_that("chemdose_chloramine_chain outputs the same as base, chemdose_chlorami
   water4 <- suppressWarnings(water_df %>%
     slice(1) %>%
     mutate(free_chlorine = 2, tot_nh3 = 2) %>%
-    define_water_chain() %>%
-    chemdose_chloramine_chain(time = 30, nh3 = 4, cl2 = 5, use_free_cl_slot = TRUE, use_tot_nh3_slot = TRUE) %>%
-    pluck_water(c("chlorinated_water"), c("ph", "free_chlorine", "combined_chlorine")))
+    define_water_df() %>%
+    chemdose_chloramine_df(time = 30, nh3 = 4, cl2 = 5, use_free_cl_slot = TRUE, use_tot_nh3_slot = TRUE, pluck_cols = TRUE))
 
-
-  expect_equal(water4$chlorinated_water_ph[1], water3@ph)
-  expect_equal(water4$chlorinated_water_free_chlorine[1], water3@free_chlorine)
-  expect_equal(water4$chlorinated_water_combined_chlorine[1], water3@combined_chlorine)
+  expect_equal(water4$chloraminated_free_chlorine[1], water3@free_chlorine)
+  expect_equal(water4$chloraminated_combined_chlorine[1], water3@combined_chlorine)
 })
 
 # Test that output is a column of water class lists, and changing the output column name works
 
-test_that("chemdose_chloramine_chain output is list of water class objects, and can handle an ouput_water arg", {
+test_that("chemdose_chloramine_df output is list of water class objects, and can handle an ouput_water arg", {
   testthat::skip_on_cran()
-  water1 <- suppressWarnings(water_df %>%
+  water1 <- water_df %>%
     slice(1) %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(input_water = "balanced_water", time = 10, nh3 = 3, cl2 = 3))
+    define_water_df() %>%
+    chemdose_chloramine_df(time = 10, nh3 = 3, cl2 = 3)
 
-  water2 <- purrr::pluck(water1, 6, 1)
+  water2 <- purrr::pluck(water1, "chloraminated", 1)
 
   water3 <- suppressWarnings(water_df %>%
-    define_water_chain() %>%
+    define_water_df() %>%
     mutate(nh3 = 3) %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(output_water = "diff_name", time = 10, cl2 = 3))
+    chemdose_chloramine_df(output_water = "diff_name", time = 10, cl2 = 3))
 
   expect_s4_class(water2, "water") # check class
-  expect_equal(names(water3[6]), "diff_name") # check if output_water arg works
+  expect_equal(names(water3[5]), "diff_name") # check if output_water arg works
 })
 
 # Check that this function can be piped to the next one
-test_that("chemdose_chloramine_chain works", {
+test_that("chemdose_chloramine_df works", {
   testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
-    define_water_chain() %>%
+    define_water_df() %>%
     mutate(
       nh3 = 2,
       cl2 = 3,
       time = 10
     ) %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(input_water = "balanced_water"))
+    chemdose_chloramine_df())
 
-  expect_equal(ncol(water1), 6) # check if pipe worked
+  expect_equal(ncol(water1), 5) # check if pipe worked
 })
 
 # Check that variety of ways to input chemicals work
-test_that("chemdose_chloramine_chain can handle different ways to input chem doses", {
+test_that("chemdose_chloramine_df can handle different ways to input chem doses", {
   testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(input_water = "balanced_water", nh3 = 3, cl2 = 5, time = 30))
+    define_water_df() %>%
+    chemdose_chloramine_df(nh3 = 3, cl2 = 5, time = 30, pluck_cols = TRUE))
 
   water2 <- suppressWarnings(water_df %>%
     mutate(tot_nh3 = 2) %>%
-    define_water_chain() %>%
+    define_water_df() %>%
     mutate(
       nh3 = 3,
       cl2 = 5,
       time = 30
     ) %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(input_water = "balanced_water"))
+    chemdose_chloramine_df(pluck_cols = TRUE))
 
   # test different ways to input chemical
-  expect_equal(
-    pluck_water(water1, "chlorinated_water", "free_chlorine")$chlorinated_water_free_chlorine,
-    pluck_water(water2, "chlorinated_water", "free_chlorine")$chlorinated_water_free_chlorine
-  )
+  expect_equal(water1$chloraminated_free_chlorine, water2$chloraminated_free_chlorine)
 
   water3 <- suppressWarnings(water_df %>%
-    define_water_chain() %>%
+    define_water_df() %>%
     mutate(nh3 = seq(0, 11, 1)) %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(cl2 = c(5, 8), time = 30))
+    chemdose_chloramine_df(cl2 = c(5, 8), time = 30, pluck_cols = TRUE))
 
   water4 <- water3 %>%
-    slice(7) %>% # same starting wq as water 5
-    pluck_water("chlorinated_water", c("combined_chlorine", "free_chlorine"))
+    slice(4) # same starting wq as water 5
 
   water5 <- water1 %>%
-    slice(4) %>% # same starting wq as water 4
-    pluck_water("chlorinated_water", c("combined_chlorine", "free_chlorine"))
+    slice(4) # same starting wq as water 4
 
   expect_equal(
-    water4$chlorinated_water_combined_chlorine,
-    water5$chlorinated_water_combined_chlorine
+    water4$chloraminated_combined_chlorine,
+    water5$chloraminated_combined_chlorine
   )
 
   expect_equal(
-    water4$chlorinated_water_free_chlorine,
-    water5$chlorinated_water_free_chlorine
+    water4$chloraminated_free_chlorine,
+    water5$chloraminated_free_chlorine
   )
 
   water6 <- suppressWarnings(water_df %>%
     mutate(tot_nh3 = 2) %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
+    define_water_df() %>%
     mutate(nh3 = 3) %>%
-    chemdose_chloramine_chain(input_water = "balanced_water", use_tot_nh3_slot = TRUE, cl2 = 5, time = 30))
+    chemdose_chloramine_df(use_tot_nh3_slot = TRUE, cl2 = 5, time = 30, pluck_cols = TRUE))
 
   water7 <- suppressWarnings(water_df %>%
     mutate(tot_nh3 = 2) %>%
-    define_water_chain() %>%
+    define_water_df() %>%
     mutate(
       nh3 = 3,
       use_tot_nh3_slot = TRUE
     ) %>%
-    balance_ions_chain() %>%
-    chemdose_chloramine_chain(input_water = "balanced_water", cl2 = 5, time = 30))
+    chemdose_chloramine_df(cl2 = 5, time = 30, pluck_cols = TRUE))
+
   # test different ways to call use_slot
-  expect_equal(
-    pluck_water(water6, "chlorinated_water", "combined_chlorine")$chlorinated_water_combined_chlorine,
-    pluck_water(water7, "chlorinated_water", "combined_chlorine")$chlorinated_water_combined_chlorine
-  )
+  expect_equal(water6$chloraminated_combined_chlorine, water7$chloraminated_combined_chlorine)
+
   # both waters 2 and 7 have starting tot_nh3, but only water 7 has use_tot_nh3_slot set to TRUE
   expect_error(
-    expect_equal(
-      pluck_water(water2, "chlorinated_water", "combined_chlorine")$chlorinated_water_combined_chlorine,
-      pluck_water(water7, "chlorinated_water", "combined_chlorine")$chlorinated_water_combined_chlorine
-    )
+    expect_equal(water2$chloraminated_combined_chlorine, water7$chloraminated_combined_chlorine)
   )
 })
 
@@ -498,36 +487,36 @@ test_that("chemdose_chloramine_chain can handle different ways to input chem dos
 # # with alpha1TOTNH <- 1/(1 + H/ks$knh4)
 # example_df1 <- water_df %>%
 #   mutate(free_chlorine = 10, tot_nh3 = 2) %>%
-#   define_water_chain() %>%
-#   balance_ions_chain() %>%
+#   define_water_df() %>%
+#   balance_ions_df() %>%
 #   mutate(
 #        time = 30,
 #        multi_cl_source = 1,
 #        multi_nh3_source = 1
 #    ) %>%
-#    chemdose_chloramine_chain(input_water = "balanced_water",
+#    chemdose_chloramine_df(input_water = "balanced_water",
 #                              cl2 = seq(2, 24, 2),
 #                              nh3 = 2)
 
 # # With alpha1TOTNH <- calculate_alpha1_ammonia(H, ks)
 # example_df2 <- water_df %>%
 #   mutate(free_chlorine = 10, tot_nh3 = 2) %>%
-#   define_water_chain() %>%
-#   balance_ions_chain() %>%
+#   define_water_df() %>%
+#   balance_ions_df() %>%
 #   mutate(
 #     time = 30,
 #     multi_cl_source = 1,
 #     multi_nh3_source = 1
 #   ) %>%
-#   chemdose_chloramine_chain(input_water = "balanced_water",
+#   chemdose_chloramine_df(input_water = "balanced_water",
 #                             cl2 = seq(2, 24, 2),
 #                             nh3 = 2)
 #
-# test1 <- pluck_water(example_df1,input_waters = c('chlorinated_water'),'tot_nh3')
-# test2 <- pluck_water(example_df2,input_waters = c('chlorinated_water'),'tot_nh3')
+# test1 <- pluck_water(example_df1,input_waters = c('chloraminated'),'tot_nh3')
+# test2 <- pluck_water(example_df2,input_waters = c('chloraminated'),'tot_nh3')
 #
-# test1 <- convert_units(test1$chlorinated_water_tot_nh3, 'n','M','mg/L')
-# test2 <- convert_units(test2$chlorinated_water_tot_nh3, 'n', 'M','mg/L')
+# test1 <- convert_units(test1$chloraminated_tot_nh3, 'n','M','mg/L')
+# test2 <- convert_units(test2$chloraminated_tot_nh3, 'n', 'M','mg/L')
 #
 # plot(test1,test2, xlim=c(0,7),ylim=c(0,7))
 #
