@@ -21,18 +21,32 @@ test_that("Decarbonate works", {
 })
 
 
-test_that("Decarbonate chain takes and returns correct argument types and classes.", {
+test_that("Decarbonate df takes and returns correct argument types and classes.", {
   testthat::skip_on_cran()
   water0 <- water_df %>%
-    define_water_chain("test")
+    define_water_df("test")
 
-  water1 <- decarbonate_ph_chain(water0, "test", "decarb", co2_removed = .9)
+  water1 <- decarbonate_ph_df(water0, "test", "decarb", co2_removed = .9)
   water2 <- water0 %>%
-    mutate(removal = .9) %>%
-    decarbonate_ph_chain("test", "decarb", co2_removed = removal)
+    dplyr::mutate(removal = .9) %>%
+    decarbonate_ph_df("test", "decarb", co2_removed = removal)
 
-  expect_error(decarbonate_ph_chain(water_df, co2_removed = .9))
-  expect_error(decarbonate_ph_chain(water0))
+  expect_error(decarbonate_ph_df(water_df, co2_removed = .9))
+  expect_error(decarbonate_ph_df(water0))
   expect_s4_class(water1$decarb[[1]], "water")
   expect_equal(water1$decarb, water2$decarb)
 })
+
+test_that("Decarbonate df pluck_cols works the same as pluck_water.", {
+  testthat::skip_on_cran()
+  water0 <- water_df %>%
+    define_water_df("test")
+
+  water1 <- decarbonate_ph_df(water0, "test", "decarb", co2_removed = .9) %>%
+    pluck_water(c("decarb"), c("ph", "alk"))
+  water2 <- decarbonate_ph_df(water0, "test", "decarb", co2_removed = .9, pluck_cols = TRUE)
+
+  expect_equal(water1$decarb_ph, water2$decarb_ph)
+  expect_equal(water1$decarb_alk, water2$decarb_alk)
+  expect_equal(ncol(water1), ncol(water2))
+})
