@@ -1,4 +1,5 @@
 # PAC_TOC ----
+library(dplyr)
 
 test_that("pac_toc returns no removed DOC value when PAC dose is 0.", {
   water1 <- suppressWarnings(define_water(doc = 2.5, uv254 = 0.05, toc = 3.5))
@@ -67,7 +68,7 @@ test_that("Error when an unaccepted PAC type is entered.", {
 ################################################################################*
 # pac_toc helpers ----
 
-test_that("pac_toc_chain outputs are the same as base function, pac_toc", {
+test_that("pac_toc_df outputs are the same as base function, pac_toc", {
   testthat::skip_on_cran()
   water0 <- define_water(7.9, 20, 50,
     tot_hard = 50, ca = 13, mg = 4,
@@ -80,18 +81,17 @@ test_that("pac_toc_chain outputs are the same as base function, pac_toc", {
 
   water2 <- water_df %>%
     slice(1) %>%
-    define_water_chain() %>%
-    pac_toc_chain(dose = 10, time = 10, output_water = "pac") %>%
-    pluck_water("pac", c("doc", "toc", "uv254"))
+    define_water_df() %>%
+    pac_toc_df(dose = 10, time = 10, output_water = "pac", pluck_cols = TRUE)
 
-  types <- tibble(type = c("wood", "lignite"))
-  doses <- tibble(PACDose = seq(10, 16, 2))
+  types <- data.frame(type = c("wood", "lignite"))
+  doses <- data.frame(PACDose = seq(10, 16, 2))
   water3 <- water_df %>%
     slice(1) %>%
-    define_water_chain() %>%
-    cross_join(types) %>%
-    cross_join(doses) %>%
-    pac_toc_chain(dose = PACDose, time = 10, output_water = "pac") %>%
+    define_water_df() %>%
+    merge(types) %>%
+    merge(doses) %>%
+    pac_toc_df(dose = PACDose, time = 10, output_water = "pac") %>%
     pluck_water("pac", "doc")
 
   water4 <- pac_toc(water0, time = 10, dose = 16, type = "lignite")
@@ -103,117 +103,115 @@ test_that("pac_toc_chain outputs are the same as base function, pac_toc", {
 
 # Test that output is a column of water class lists, and changing the output column name works
 
-test_that("pac_toc_chain output is list of water class objects, and can handle an ouput_water arg", {
+test_that("pac_toc_df output is list of water class objects, and can handle an ouput_water arg", {
   testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
     slice(1) %>%
-    define_water_chain("raw") %>%
-    pac_toc_chain(input_water = "raw", time = 10, dose = 4))
+    define_water_df("raw") %>%
+    pac_toc_df(input_water = "raw", time = 10, dose = 4))
 
-  water2 <- purrr::pluck(water1, "pac_water", 1)
+  water2 <- purrr::pluck(water1, "paced", 1)
 
   water3 <- suppressWarnings(water_df %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(
       dose = 4,
       time = 10
     ) %>%
-    pac_toc_chain(input_water = "raw", output_water = "diff_name"))
+    pac_toc_df(input_water = "raw", output_water = "diff_name"))
 
   expect_s4_class(water2, "water") # check class
   expect_true(exists("diff_name", water3)) # check if output_water arg works
 })
 
-# Check pac_toc_chain can use a column or function argument for chemical dose
+# Check pac_toc_df can use a column or function argument for chemical dose
 
-test_that("pac_toc_chain can use a column or function argument for chemical dose", {
+test_that("pac_toc_df can use a column or function argument for chemical dose", {
   testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
     slice(1) %>%
-    define_water_chain("raw") %>%
-    pac_toc_chain(input_water = "raw", time = 50, dose = 10) %>%
-    pluck_water("pac_water", c("doc", "uv254")))
+    define_water_df("raw") %>%
+    pac_toc_df(input_water = "raw", time = 50, dose = 10, pluck_cols = TRUE))
 
   water2 <- suppressWarnings(water_df %>%
     slice(1) %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(
       time = 50,
-      dose = 10,
+      dose = 10
     ) %>%
-    pac_toc_chain(input_water = "raw") %>%
-    pluck_water("pac_water", c("doc", "uv254")))
+    pac_toc_df(input_water = "raw", pluck_cols = TRUE))
 
+  # test that pluck_col does the same as pluck_water
   water3 <- suppressWarnings(water_df %>%
     slice(1) %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(time = 50) %>%
-    pac_toc_chain(input_water = "raw", dose = 10) %>%
-    pluck_water("pac_water", c("doc", "uv254")))
+    pac_toc_df(input_water = "raw", dose = 10) %>%
+    pluck_water("paced", c("toc", "doc", "uv254")))
 
-  expect_equal(water1$pac_water_doc, water2$pac_water_doc) # test different ways to input args
-  expect_equal(water1$pac_water_uv254, water2$pac_water_uv254)
+  expect_equal(water1$paced_doc, water2$paced_doc) # test different ways to input args
+  expect_equal(water1$paced_uv254, water2$paced_uv254)
 
   # Test that inputting time/dose separately (in column and as an argument)  gives same results
-  expect_equal(water1$pac_water_doc, water3$pac_water_doc)
+  expect_equal(water1$paced_doc, water3$paced_doc)
 
 
   water4 <- water_df %>%
     slice(1:4) %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(time = c(20, 20, 50, 50)) %>%
-    pac_toc_chain(input_water = "raw", output_water = "pac", dose = c(10, 20))
-  water4b <- water4 %>%
-    filter(dose == 10)
+    pac_toc_df(input_water = "raw", output_water = "pac", dose = c(10, 20))
+  water4b <- water4[water4$dose == 10, ]
 
   water5 <- water_df %>%
     slice(1:4) %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(PACtime = c(20, 20, 50, 50)) %>%
-    pac_toc_chain(input_water = "raw", output_water = "pac", dose = c(10, 20), time = PACtime)
+    pac_toc_df(input_water = "raw", output_water = "pac", dose = c(10, 20), time = PACtime)
 
   water6 <- water_df %>%
     slice(1:4) %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(time = c(20, 20, 50, 50)) %>%
-    cross_join(tibble(dose = c(10, 20))) %>%
-    pac_toc_chain(input_water = "raw", output_water = "pac")
+    merge(data.frame(dose = c(10, 20))) %>%
+    pac_toc_df(input_water = "raw", output_water = "pac")
 
   water7 <- water_df %>%
     slice(1:4) %>%
-    define_water_chain("raw") %>%
+    define_water_df("raw") %>%
     mutate(
       PACtime = c(20, 20, 50, 50),
       type = "bituminous"
     ) %>%
-    pac_toc_chain(input_water = "raw", output_water = "pac", dose = 10, time = PACtime)
+    pac_toc_df(input_water = "raw", output_water = "pac", dose = 10, time = PACtime)
 
   expect_equal(water4$pac, water5$pac)
   expect_equal(water4$pac, water6$pac)
   expect_equal(water4b$pac, water7$pac)
 })
 
-test_that("pac_toc_chain errors with argument + column for same param", {
+test_that("pac_toc_df errors with argument + column for same param", {
   testthat::skip_on_cran()
   water <- water_df %>%
-    define_water_chain("raw")
+    define_water_df("raw")
   expect_error(water %>%
     mutate(dose = 5) %>%
-    pac_toc_chain(input_water = "raw", time = 50, dose = 10))
+    pac_toc_df(input_water = "raw", time = 50, dose = 10))
   expect_error(water %>%
     mutate(time = 5) %>%
-    pac_toc_chain(input_water = "raw", time = 50, dose = 10))
+    pac_toc_df(input_water = "raw", time = 50, dose = 10))
 })
 
-test_that("pac_toc_chain correctly handles arguments with multiple numbers", {
+test_that("pac_toc_df correctly handles arguments with multiple numbers", {
   testthat::skip_on_cran()
   water <- water_df %>%
-    define_water_chain("raw")
+    define_water_df("raw")
 
   water1 <- water %>%
-    pac_toc_chain("raw", time = c(10, 20), dose = 5)
+    pac_toc_df("raw", time = c(10, 20), dose = 5)
   water2 <- water %>%
-    pac_toc_chain("raw", time = 20, dose = seq(10, 30, 10))
+    pac_toc_df("raw", time = 20, dose = seq(10, 30, 10))
 
   expect_equal(nrow(water) * 2, nrow(water1))
   expect_equal(nrow(water) * 3, nrow(water2))
