@@ -1,4 +1,6 @@
 # BASE FUNCTION ----
+library(dplyr)
+
 test_that("solveresid_o3 returns the input residual when time is 0, or an error when time is missing.", {
   water1 <- suppressWarnings(define_water(7.5, 20, 66, toc = 4, uv254 = .2, br = 30))
 
@@ -38,67 +40,61 @@ test_that("solveresid_o3 works.", {
 
 
 # HELPERS ----
-test_that("solveresid_o3_once outputs are the same as base function, solveresid_o3", {
+test_that("solveresid_o3_df outputs are the same as base function, solveresid_o3", {
   testthat::skip_on_cran()
   water1 <- suppressWarnings(define_water(
     ph = 7.9, temp = 20, alk = 50, tot_hard = 50, na = 20, k = 20,
     cl = 30, so4 = 20, tds = 200, cond = 100, toc = 2, doc = 1.8, uv254 = 0.05, br = 50
   )) %>%
-    balance_ions() %>%
     solveresid_o3(time = 30, dose = 5)
 
-  water2 <- suppressWarnings(water_df %>%
+  water2 <- water_df %>%
     slice(1) %>%
     mutate(br = 50) %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
-    solveresid_o3_once(input_water = "balanced_water", time = 30, dose = 5))
+    define_water_df() %>%
+    solveresid_o3_df(time = 30, dose = 5)
 
   expect_equal(water1, water2$o3resid)
 })
 
 # Check that output is a data frame
 
-test_that("solveresid_o3_once is a data frame", {
+test_that("solveresid_o3_df is a data frame", {
   testthat::skip_on_cran()
-  water1 <- suppressWarnings(water_df %>%
+  water1 <- water_df %>%
     slice(1) %>%
     mutate(br = 50) %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
-    solveresid_o3_once(input_water = "balanced_water", time = 30, dose = 5))
+    define_water_df() %>%
+    solveresid_o3_df(time = 30, dose = 5)
 
 
   expect_true(is.data.frame(water1))
 })
 
-# Check solveresid_o3_once can use a column or function argument for chemical dose
+# Check solveresid_o3_df can use a column or function argument for chemical dose
 
-test_that("solveresid_o3_once can use a column and/or function argument for time and dose", {
+test_that("solveresid_o3_df can use a column and/or function argument for time and dose", {
   testthat::skip_on_cran()
   water0 <- water_df %>%
-    define_water_chain()
+    define_water_df()
 
   time <- data.frame(time = seq(2, 24, 2))
-  water1 <- suppressWarnings(water_df %>%
+  water1 <- water_df %>%
     mutate(br = 50) %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
-    cross_join(time) %>%
-    solveresid_o3_once(input_water = "balanced_water", dose = 5))
+    define_water_df() %>%
+    merge(time) %>%
+    solveresid_o3_df(dose = 5)
 
-  water2 <- suppressWarnings(water_df %>%
+  water2 <- water_df %>%
     mutate(br = 50) %>%
-    define_water_chain() %>%
-    balance_ions_chain() %>%
-    solveresid_o3_once(input_water = "balanced_water", time = seq(2, 24, 2), dose = 5) %>%
-    unique())
+    define_water_df() %>%
+    solveresid_o3_df(time = seq(2, 24, 2), dose = 5)
 
   water3 <- water_df %>%
     mutate(br = 50) %>%
-    define_water_chain() %>%
-    cross_join(time) %>%
-    solveresid_o3_once(dose = c(5, 8))
+    define_water_df() %>%
+    merge(time) %>%
+    solveresid_o3_df(dose = c(5, 8))
 
   expect_equal(water1$o3resid, water2$o3resid) # test different ways to input time
   expect_equal(ncol(water3), ncol(water0) + 3) # adds cols for time, dose, and o3resid
